use std::sync::Arc;

use reqwest::{Client, Url};
use serde::Deserialize;

/// Shared mutable state object
#[derive(Debug)]
pub struct State {
    pub offset: u32,
}

/// Shared config object
#[derive(Debug, Clone)]
pub struct Config {
    pub client: Client,
    pub upd_url: Arc<Url>,
    pub send_url: Arc<Url>,
}

/// Object holding a context for fn send_comic
#[derive(Debug)]
pub struct SenderCtx {
    pub msg: TgMessage,
    pub url: String,
    pub comic_id: u16,
}

/// Telegram Bot API response:
/// `{ok: true, result: (what we actually need)}`
#[derive(Debug, Deserialize)]
pub struct TgResponse {
    pub result: Vec<TgUpdate>,
}

/// Telegram Bot API update event schema
#[derive(Debug, Deserialize)]
pub struct TgUpdate {
    pub update_id: u32,
    pub message: Option<TgMessage>,
}

/// Telegram Bot API message object schema
#[derive(Debug, Deserialize)]
pub struct TgMessage {
    #[serde(rename = "message_thread_id")]
    pub thread: Option<i32>,
    pub chat: TgChat,
    pub text: Option<String>,
}

/// Telegram Bot API chat object schema
#[derive(Debug, Deserialize)]
pub struct TgChat {
    pub id: i64,
}

/// XKCD API comic info schema
#[derive(Debug, Deserialize)]
pub struct XkcdInfo {
    pub title: String, // same in two APIs
    #[serde(alias = "image")] // xkcd.ru
    pub img: String, // xkcd.com
    #[serde(alias = "text")] // xkcd.ru
    pub alt: String, // xkcd.com
}
